# ----------------------------------
# Options affecting listfile parsing
# ----------------------------------
with section("parse"):
    # Specify structure for custom cmake functions
    additional_commands = {
        "foo": {
            "flags": ["BAR", "BAZ"],
            "kwargs": {"DEPENDS": "*", "HEADERS": "*", "SOURCES": "*"},
        }
    }

    # Override configurations per-command where available
    override_spec = {}

    # Specify variable tags.
    vartags = []

    # Specify property tags.
    proptags = []

# -----------------------------
# Options affecting formatting.
# -----------------------------
with section("format"):
    # Disable formatting entirely, making cmake-format a no-op
    disable = False

    # How wide to allow formatted cmake files
    line_width = 80

    # How many spaces to tab for indent
    tab_size = 2

    # If true, lines are indented using tab characters (utf-8 0x09) instead of
    # <tab_size> space characters (utf-8 0x20). In cases where the layout would
    # require a fractional tab character, the behavior of the  fractional
    # indentation is governed by <fractional_tab_policy>
    use_tabchars = False

    # If <use_tabchars> is True, then the value of this variable indicates how
    # fractional indentions are handled during whitespace replacement. If set to
    # 'use-space', fractional indentation is left as spaces (utf-8 0x20). If set
    # to `round-up` fractional indentation is replaced with a single tab character
    # (utf-8 0x09) effectively shifting the column to the next tabstop
    fractional_tab_policy = "use-space"

    # If an argument group contains more than this many sub-groups (parg or kwarg
    # groups) then force it to a vertical layout.
    max_subgroups_hwrap = 2

    # If a positional argument group contains more than this many arguments, then
    # force it to a vertical layout.
    max_pargs_hwrap = 6

    # If a cmdline positional group consumes more than this many lines without
    # nesting, then invalidate the layout (and nest)
    max_rows_cmdline = 2

    # If true, separate flow control names from their parentheses with a space
    separate_ctrl_name_with_space = False

    # If true, separate function names from parentheses with a space
    separate_fn_name_with_space = False

    # If a statement is wrapped to more than one line, than dangle the closing
    # parenthesis on its own line.
    dangle_parens = False

    # If the trailing parenthesis must be 'dangled' on its on line, then align it
    # to this reference: `prefix`: the start of the statement,  `prefix-indent`:
    # the start of the statement, plus one indentation  level, `child`: align to
    # the column of the arguments
    dangle_align = "prefix"

    # If the statement spelling length (including space and parenthesis) is
    # smaller than this amount, then force reject nested layouts.
    min_prefix_chars = 4

    # If the statement spelling length (including space and parenthesis) is larger
    # than the tab width by more than this amount, then force reject un-nested
    # layouts.
    max_prefix_chars = 10

    # If a candidate layout is wrapped horizontally but it exceeds this many
    # lines, then reject the layout.
    max_lines_hwrap = 2

    # What style line endings to use in the output.
    line_ending = "unix"

    # Format command names consistently as 'lower' or 'upper' case
    command_case = "canonical"

    # Format keywords consistently as 'lower' or 'upper' case
    keyword_case = "unchanged"

    # A list of command names which should always be wrapped
    always_wrap = []

    # If true, the argument lists which are known to be sortable will be sorted
    # lexicographicall
    enable_sort = True

    # If true, the parsers may infer whether or not an argument list is sortable
    # (without annotation).
    autosort = False

    # By default, if cmake-format cannot successfully fit everything into the
    # desired linewidth it will apply the last, most agressive attempt that it
    # made. If this flag is True, however, cmake-format will print error, exit
    # with non-zero status code, and write-out nothing
    require_valid_layout = False

    # A dictionary mapping layout nodes to a list of wrap decisions. See the
    # documentation for more information.
    layout_passes = {}

# ------------------------------------------------
# Options affecting comment reflow and formatting.
# ------------------------------------------------
with section("markup"):
    # What character to use for bulleted lists
    bullet_char = "*"

    # What character to use as punctuation after numerals in an enumerated list
    enum_char = "."

    # If comment markup is enabled, don't reflow the first comment block in each
    # listfile. Use this to preserve formatting of your copyright/license
    # statements.
    first_comment_is_literal = False

    # If comment markup is enabled, don't reflow any comment block which matches
    # this (regex) pattern. Default is `None` (disabled).
    literal_comment_pattern = None

    # Regular expression to match preformat fences in comments default=
    # ``r'^\s*([`~]{3}[`~]*)(.*)$'``
    fence_pattern = "^\\s*([`~]{3}[`~]*)(.*)$"

    # Regular expression to match rulers in comments default=
    # ``r'^\s*[^\w\s]{3}.*[^\w\s]{3}$'``
    ruler_pattern = "^\\s*[^\\w\\s]{3}.*[^\\w\\s]{3}$"

    # If a comment line matches starts with this pattern then it is explicitly a
    # trailing comment for the preceeding argument. Default is '#<'
    explicit_trailing_pattern = "#<"

    # If a comment line starts with at least this many consecutive hash
    # characters, then don't lstrip() them off. This allows for lazy hash rulers
    # where the first hash char is not separated by space
    hashruler_min_length = 10

    # If true, then insert a space between the first hash char and remaining hash
    # chars in a hash ruler, and normalize its length to fill the column
    canonicalize_hashrulers = True

    # enable comment markup parsing and reflow
    enable_markup = False

# ----------------------------
# Options affecting the linter
# ----------------------------
with section("lint"):
    # a list of lint codes to disable
    disabled_codes = []

    # regular expression pattern describing valid function names
    function_pattern = "[0-9a-z_]+"

    # regular expression pattern describing valid macro names
    macro_pattern = "[0-9A-Z_]+"

    # regular expression pattern describing valid names for variables with global
    # (cache) scope
    global_var_pattern = "[A-Z][0-9A-Z_]+"

    # regular expression pattern describing valid names for variables with global
    # scope (but internal semantic)
    internal_var_pattern = "_[A-Z][0-9A-Z_]+"

    # regular expression pattern describing valid names for variables with local
    # scope
    local_var_pattern = "[a-z][a-z0-9_]+"

    # regular expression pattern describing valid names for privatedirectory
    # variables
    private_var_pattern = "_[0-9a-z_]+"

    # regular expression pattern describing valid names for public directory
    # variables
    public_var_pattern = "[A-Z][0-9A-Z_]+"

    # regular expression pattern describing valid names for function/macro
    # arguments and loop variables.
    argument_var_pattern = "[a-z][a-z0-9_]+"

    # regular expression pattern describing valid names for keywords used in
    # functions or macros
    keyword_pattern = "[A-Z][0-9A-Z_]+"

    # In the heuristic for C0201, how many conditionals to match within a loop in
    # before considering the loop a parser.
    max_conditionals_custom_parser = 2

    # Require at least this many newlines between statements
    min_statement_spacing = 1

    # Require no more than this many newlines between statements
    max_statement_spacing = 2
    max_returns = 6
    max_branches = 12
    max_arguments = 5
    max_localvars = 15
    max_statements = 50

# -------------------------------
# Options affecting file encoding
# -------------------------------
with section("encode"):
    # If true, emit the unicode byte-order mark (BOM) at the start of the file
    emit_byteorder_mark = False

    # Specify the encoding of the input file. Defaults to utf-8
    input_encoding = "utf-8"

    # Specify the encoding of the output file. Defaults to utf-8. Note that cmake
    # only claims to support utf-8 so be careful when using anything else
    output_encoding = "utf-8"

# -------------------------------------
# Miscellaneous configurations options.
# -------------------------------------
with section("misc"):
    # A dictionary containing any per-command configuration overrides. Currently
    # only `command_case` is supported.
    per_command = {}



import sys

import matplotlib.pyplot as plt
import numpy

import aoflagger as aof

nch = 256
ntimes = 1000
npol = 2

aoflagger = aof.AOFlagger()
path = aoflagger.find_strategy_file(aof.TelescopeId.Generic)
strategy = aoflagger.load_strategy_file(path)
data = aoflagger.make_image_set(ntimes, nch, npol * 2)

# Several consecutive values at the same frequency are increased
# in amplitude to simulate a RFI source. These values define
# the channel and the start and duration of the added signal.
rfi_y = int(nch * 0.3)
rfi_x_start = int(ntimes * 0.2)
rfi_x_end = int(ntimes * 0.4)
rfi_strength = 1  # 1 sigma above the noise

for imgindex in range(npol * 2):
    # Initialize data with random numbers
    values = numpy.random.normal(0, 1, [nch, ntimes])
    # Add fake transmitter
    values[rfi_y, rfi_x_start:rfi_x_end] = (
        values[rfi_y, rfi_x_start:rfi_x_end] + rfi_strength
    )
    data.set_image_buffer(imgindex, values)


flags = strategy.run(data)
flagvalues = flags.get_buffer()
flagvalues = flagvalues * 1

plt.imshow(values, cmap="viridis")
plt.colorbar()
plt.show()

plt.imshow(flagvalues, cmap="viridis")
plt.colorbar()
plt.show()



import copy

import numpy

import aoflagger


def flag(input):
    # Values below can be tweaked
    flag_polarizations = input.polarizations()
    flag_representations = [aoflagger.ComplexRepresentation.AmplitudePart]

    iteration_count = 3
    threshold_factor_step = 2.0
    base_threshold = 1.4

    # Use above values to calculate thresholds in each iteration
    r = range((iteration_count - 1), 0, -1)
    threshold_factors = numpy.power(threshold_factor_step, r)

    inpPolarizations = input.polarizations()
    input.clear_mask()

    for polarization in flag_polarizations:
        data = input.convert_to_polarization(polarization)

        for representation in flag_representations:
            data = data.convert_to_complex(representation)
            original_image = copy.copy(data)

            for threshold_factor in threshold_factors:
                # print('Flagging polarization ' + str(polarization) + ' (' + str(representation) + ', ' + str(threshold_factor) + ')')

                thr = threshold_factor * base_threshold
                aoflagger.sumthreshold(data, thr, thr, True, True)
                chdata = copy.copy(data)
                aoflagger.threshold_timestep_rms(data, 3.5)
                aoflagger.threshold_channel_rms(
                    chdata, 3.0 * threshold_factor, True
                )
                data.join_mask(chdata)

                data.set_image(original_image)
                resized_data = aoflagger.shrink(data, 3, 3)
                aoflagger.low_pass_filter(resized_data, 21, 31, 1.6, 2.2)
                aoflagger.enlarge(resized_data, data, 3, 3)
                data = original_image - data

            aoflagger.sumthreshold(
                data, base_threshold, base_threshold, True, True
            )

        if polarization in inpPolarizations:
            data = data.make_complex()
            input.set_polarization_data(polarization, data)
        else:
            input.join_mask(polarization, data)

    aoflagger.scale_invariant_rank_operator(input, 0.2, 0.2)
    aoflagger.threshold_timestep_rms(input, 4.0)


def test_sumthreshold(input):
    # Values below can be tweaked
    flag_polarizations = input.polarizations()
    flag_representations = [aoflagger.ComplexRepresentation.AmplitudePart]

    iteration_count = 3
    threshold_factor_step = 2.0
    base_threshold = 1.4

    # Use above values to calculate thresholds in iteration
    r = range((iteration_count - 1), 0, -1)
    threshold_factors = numpy.power(threshold_factor_step, r)

    inpPolarizations = input.polarizations()
    input.clear_mask()

    for polarization in flag_polarizations:
        data = input.convert_to_polarization(polarization)
        for representation in flag_representations:
            data = data.convert_to_complex(representation)
            aoflagger.sumthreshold(data, base_threshold, True, True)

        if polarization in inpPolarizations:
            data = data.make_complex()
            input.set_polarization_data(polarization, data)
        else:
            input.join_mask(polarization, data)


aoflagger.set_flag_function(flag)

print("strategy.py parsed")



import re
import sys
from subprocess import check_call, check_output

import pytest

# Append current directory to system path in order to import testconfig
sys.path.append(".")
import testconfig as config
from utils import assert_taql, untar_ms

"""
Basic integration test.

Script should be invoked like
  cd $BUILD_DIR/test/integration
  pytest ../../../test/integration/tBase.py
"""

MS = f"{config.RESOURCE_DIR}/base/3C196_spw5_sub1-WSRT.MS"


def test():
    # Prepare and validate input.
    # Note the uploaded data set should already be in this state, but this
    # guards against other tests modifying the data set.
    check_call([config.taql, "update", MS, "set", "FLAG=False"])
    check_call([config.aoquality, "collect", MS])
    result = check_output([config.aoquality, "summarizerfi", MS])
    assert result == b"4703949275\t4703953055\t146.85\t149.34\t0\n"
    check_call([config.taql, "delete", "from", f"{MS}/HISTORY"])
    assert_taql(f"select from {MS}/HISTORY")

    # Run Aoflagger and validate output.
    check_call([config.aoflagger, MS])
    check_call([config.aoquality, "collect", MS])
    result = check_output([config.aoquality, "summarizerfi", MS])
    # In the prepare step the code expects exactly 0% of the data flagged.
    # For this test we want to validate an approximate value:
    # - avoids difference between platform's floating point types to cause
    #   false positives,
    # - avoids small changes in the code to directly fail the test.
    assert re.match(
        b"4703949275\t4703953055\t146.85\t149.34\t19.5[0-9]+\n", result
    )
    # Did we write one history record?
    assert_taql(f"select from {MS}/HISTORY", 1)

    # Only test gui components if they have been built
    if config.enable_gui == "ON":
        # Test rfigui save baseline functionality (parameters: filename, a1, a2, band, sequence)
        # The output isn't checked -- this is just to see if the command succeeds.
        check_call(
            [
                config.rfigui,
                MS,
                "-save-baseline",
                "baseline-test.pdf",
                "0",
                "1",
                "0",
                "0",
            ]
        )

        # Test if the statistic plots (pdf files) can be saved.
        check_call([config.aoqplot, MS, "-save", "test", "StandardDeviation"])



import re
import sys
from subprocess import check_call, check_output

import pytest

# Append current directory to system path in order to import testconfig
sys.path.append(".")
import testconfig as config
from utils import assert_taql, untar_ms

"""
Basic integration test.

Script should be invoked like
  cd $BUILD_DIR/test/integration
  pytest ../../../test/integration/tConcatenateFrequency.py
"""

MSs = [
    f"{config.RESOURCE_DIR}/concatenate_frequency/L228163_SB150_uv.dppp.MS",
    f"{config.RESOURCE_DIR}/concatenate_frequency/L228163_SB151_uv.dppp.MS",
]


@pytest.mark.parametrize(
    "read_mode",
    ["", "-direct-read", "-indirect-read", "-memory-read", "-auto-read-mode"],
)
def test(read_mode):
    # Prepare and validate input.
    # Note the uploaded data set should already be in this state, but this
    # guards against other tests modifying the data set.
    for MS in MSs:
        check_call([config.taql, "update", MS, "set", "FLAG=False"])
        check_call([config.aoquality, "collect", MS])
        result = check_output([config.aoquality, "summarizerfi", MS])
        # Only validate the flagged percentage
        assert re.match(b".*\t0\n", result)
        check_call([config.taql, "delete", "from", f"{MS}/HISTORY"])
        assert_taql(f"select from {MS}/HISTORY")

    # Run Aoflagger and validate output.
    # The MSs are passed in reversed order to check that aoflagger sorts
    # them along frequency before concatenating
    if read_mode != "":
        check_call(
            [config.aoflagger, "-concatenate-frequency", read_mode] + MSs[::-1]
        )
    else:
        check_call([config.aoflagger, "-concatenate-frequency"] + MSs[::-1])

    for MS in MSs:
        check_call([config.aoquality, "collect", MS])
        # Did we write one history record?
        assert_taql(f"select from {MS}/HISTORY", 1)

    # In the prepare step the code expects exactly 0% of the data flagged.
    # For this test we want to validate an approximate value:
    # - avoids difference between platform's floating point types to cause
    #   false positives,
    # - avoids small changes in the code to directly fail the test.
    # Don't validate the other fields in this test.
    result = check_output([config.aoquality, "summarizerfi", MSs[0]])
    assert re.match(b".*\t0.638[0-9]+\n", result)
    result = check_output([config.aoquality, "summarizerfi", MSs[1]])
    assert re.match(b".*\t1.061[0-9]+\n", result)


# The output has been validated by running AOFlagger twice on the same data:
# 1. intervals 0 - 75
# 2. intervals 75 - 150
# The rest of the test is the same as the original test so comments have been removed.
@pytest.mark.parametrize(
    "chunk_size",
    ["75", "100", "149"],
)
def test_two_chunks(chunk_size):
    for MS in MSs:
        check_call([config.taql, "update", MS, "set", "FLAG=False"])
        check_call([config.aoquality, "collect", MS])
        result = check_output([config.aoquality, "summarizerfi", MS])

    check_call(
        [config.aoflagger, "-concatenate-frequency", "-chunk-size", chunk_size]
        + MSs
    )

    for MS in MSs:
        check_call([config.aoquality, "collect", MS])

    result = check_output([config.aoquality, "summarizerfi", MSs[0]])
    assert re.match(b".*\t0.569[0-9]+\n", result)
    result = check_output([config.aoquality, "summarizerfi", MSs[1]])
    assert re.match(b".*\t0.984[0-9]+\n", result)


# Like the previous, but using 3 chunks and limit the interval used.
# The three intervals used are:
# 1. interval 30 - 53
# 2. interval 53 - 76
# 3. interval 76 - 99
@pytest.mark.parametrize(
    "chunk_size",
    ["23", "34"],
)
def test_three_chunks_limited_interval(chunk_size):
    for MS in MSs:
        check_call([config.taql, "update", MS, "set", "FLAG=False"])
        check_call([config.aoquality, "collect", MS])
        result = check_output([config.aoquality, "summarizerfi", MS])

    check_call(
        [
            config.aoflagger,
            "-concatenate-frequency",
            "-interval",
            "30",
            "99",
            "-chunk-size",
            chunk_size,
        ]
        + MSs
    )

    for MS in MSs:
        check_call([config.aoquality, "collect", MS])

    result = check_output([config.aoquality, "summarizerfi", MSs[0]])
    assert re.match(b".*\t0.175[0-9]+\n", result)
    result = check_output([config.aoquality, "summarizerfi", MSs[1]])
    assert re.match(b".*\t0.384[0-9]+\n", result)



import sys

import numpy
import pytest

import aoflagger

# Append current directory to system path in order to import testconfig
sys.path.append(".")
import testconfig as config

"""
Use AOFlagger's Python interface to perform a simple flagging of Gaussian data.
This test does not use an initial mask.
"""


def test_without_initial_mask():
    n_channels = 256
    n_times = 400
    n_polarizations = 4
    n_images_per_polarization = 2  # Real and imaginary
    n_images = n_polarizations * n_images_per_polarization

    flagger = aoflagger.AOFlagger()
    path = flagger.find_strategy_file(aoflagger.TelescopeId.Generic)
    strategy = flagger.load_strategy_file(path)
    data = flagger.make_image_set(n_times, n_channels, n_images)

    for imgindex in range(n_images):
        # Initialize data with random numbers
        values = numpy.random.normal(0, 1, [n_channels, n_times])
        data.set_image_buffer(imgindex, values)

    flags = strategy.run(data)
    flagvalues = flags.get_buffer()
    ratio = float(sum(sum(flagvalues))) / (n_channels * n_times)

    # This tests the false positive ratio, which should be around 0.5%.
    assert ratio < 0.02

    # With so many samples, always at least a few samples should get flagged:
    assert ratio > 0


def test_set_mask():
    flagger = aoflagger.AOFlagger()
    n_channels = 100
    n_times = 50
    flag_mask = flagger.make_flag_mask(n_channels, n_times, False)
    buf = flag_mask.get_buffer()
    assert not buf.any()

    buf[10, 20] = True
    flag_mask.set_buffer(buf)
    buf = flag_mask.get_buffer()
    assert sum(sum(buf)) == 1
    assert buf[10, 20] == True



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) 2016 The Pybind Development Team, All rights reserved.

# Setup.py that calls CMake. Largely taken from https://github.com/pybind/cmake_example/

import os
import re
import shutil
import subprocess
import sys
from glob import glob

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext


# A CMakeExtension needs a sourcedir instead of a file list.
# The name must be the _single_ output extension from the CMake build.
# If you need multiple extensions, see scikit-build.
class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=""):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def build_extension(self, ext):
        extdir = os.path.abspath(
            os.path.dirname(self.get_ext_fullpath(ext.name))
        )

        # required for auto-detection & inclusion of auxiliary "native" libs
        if not extdir.endswith(os.path.sep):
            extdir += os.path.sep

        debug = (
            int(os.environ.get("DEBUG", 0))
            if self.debug is None
            else self.debug
        )
        cfg = "Debug" if debug else "Release"

        # CMake lets you override the generator - we need to check this.
        # Can be set with Conda-Build, for example.
        cmake_generator = os.environ.get("CMAKE_GENERATOR", "")

        # Set Python_EXECUTABLE instead if you use PYBIND11_FINDPYTHON

        cmake_args = [
            f"-DPYTHON_EXECUTABLE={sys.executable}",
            f"-DCMAKE_BUILD_TYPE={cfg}",  # not used on MSVC, but no harm
            "-DENABLE_GUI=False",
            "-DPORTABLE=True",
            "-DGIT_SUBMODULE=Off",
        ]
        build_args = []
        # Adding CMake arguments set as environment variable
        # (needed e.g. to build for ARM OSx on conda-forge)
        if "CMAKE_ARGS" in os.environ:
            cmake_args += [
                item for item in os.environ["CMAKE_ARGS"].split(" ") if item
            ]

        # Using Ninja-build since it a) is available as a wheel and b)
        # multithreads automatically. MSVC would require all variables be
        # exported for Ninja to pick it up, which is a little tricky to do.
        # Users can override the generator with CMAKE_GENERATOR in CMake
        # 3.15+.
        if not cmake_generator or cmake_generator == "Ninja":
            try:
                import ninja  # noqa: F401

                ninja_executable_path = os.path.join(ninja.BIN_DIR, "ninja")
                cmake_args += [
                    "-GNinja",
                    f"-DCMAKE_MAKE_PROGRAM:FILEPATH={ninja_executable_path}",
                ]
            except ImportError:
                pass

        if sys.platform.startswith("darwin"):
            # Cross-compile support for macOS - respect ARCHFLAGS if set
            archs = re.findall(r"-arch (\S+)", os.environ.get("ARCHFLAGS", ""))
            if archs:
                cmake_args += [
                    "-DCMAKE_OSX_ARCHITECTURES={}".format(";".join(archs))
                ]

        # Set CMAKE_BUILD_PARALLEL_LEVEL to control the parallel build level
        # across all generators.
        if "CMAKE_BUILD_PARALLEL_LEVEL" not in os.environ:
            # self.parallel is a Python 3 only way to set parallel jobs by hand
            # using -j in the build_ext call, not supported by pip or PyPA-build.
            if hasattr(self, "parallel") and self.parallel:
                # CMake 3.12+ only.
                build_args += [f"-j{self.parallel}"]

        build_temp = os.path.join(self.build_temp, ext.name)
        if not os.path.exists(build_temp):
            os.makedirs(build_temp)

        subprocess.check_call(
            ["cmake", ext.sourcedir] + cmake_args, cwd=build_temp
        )
        subprocess.check_call(
            ["cmake", "--build", ".", "--target", "python_aoflagger"]
            + build_args,
            cwd=build_temp,
        )

        if not os.path.exists(extdir):
            os.makedirs(extdir)
            shutil.move(
                glob(
                    os.path.join(
                        build_temp, "python", "aoflagger.cpython-*.so"
                    )
                )[0],
                extdir,
            )


# The information here can also be placed in setup.cfg - better separation of
# logic and declaration, and simpler if you include description/version in a file.
setup(
    name="aoflagger",
    version=re.search(
        "AOFLAGGER_VERSION_STR +([0-9\.]*)",
        open("CMakeVersionInfo.txt", "r").read(),
    ).groups()[0],
    author="André Offringa",
    author_email="offringa@astron.nl",
    description="AOFlagger",
    long_description="The AOFlagger is a tool that can find and remove radio-frequency interference (RFI) in radio astronomical observations.",
    ext_modules=[CMakeExtension("aoflagger")],
    cmdclass={"build_ext": CMakeBuild},
    zip_safe=False,
    python_requires=">=3.6",
    data_files=[
        (
            "../../aoflagger.libs/aoflagger/strategies",
            glob("data/strategies/*.lua"),
        ),
    ],
    url="https://aoflagger.readthedocs.io/",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Topic :: Scientific/Engineering :: Astronomy",
    ],
)



import sys

import numpy

import aoflagger

nch = 256
ntimes = 1000
count = 50  # number of trials in the false-positives test

flagger = aoflagger.AOFlagger()
path = flagger.find_strategy_file(aoflagger.TelescopeId.Generic)
strategy = flagger.load_strategy_file(path)
data = flagger.make_image_set(ntimes, nch, 8)

ratiosum = 0.0
ratiosumsq = 0.0
for repeat in range(count):
    for imgindex in range(8):
        # Initialize data with random numbers
        values = numpy.random.normal(0, 1, [nch, ntimes])
        data.set_image_buffer(imgindex, values)

    flags = strategy.run(data)
    flagvalues = flags.get_buffer()
    ratio = float(sum(sum(flagvalues))) / (nch * ntimes)
    ratiosum += ratio
    ratiosumsq += ratio * ratio
    sys.stdout.write(".")
    sys.stdout.flush()

print("")

print(
    "Percentage flags (false-positive rate) on Gaussian data: "
    + str(ratiosum * 100.0 / count)
    + "% +/- "
    + str(
        numpy.sqrt(
            (ratiosumsq / count - ratiosum * ratiosum / (count * count))
        )
        * 100.0
    )
)



import numpy

import aoflagger as aof

print("Flagging with AOFlagger version " + aof.AOFlagger.get_version_string())

nch = 256
ntimes = 1000

aoflagger = aof.AOFlagger()

# Load strategy from disk (alternatively use 'make_strategy' to use a default one)
path = aoflagger.find_strategy_file(aof.TelescopeId.Generic)
strategy = aoflagger.load_strategy_file(path)

data = aoflagger.make_image_set(ntimes, nch, 8)

print("Number of times: " + str(data.width()))
print("Number of channels: " + str(data.height()))

# When flagging multiple baselines, iterate over the baselines and
# call the following code for each baseline
# (to use multithreading, make sure to create an imageset for each
# thread)

# Make eight images: real and imaginary for 4 pol

for imgindex in range(8):
    # Initialize data
    values = numpy.zeros([nch, ntimes])
    data.set_image_buffer(imgindex, values)

flags = strategy.run(data)
flagvalues = flags.get_buffer()
flagcount = sum(sum(flagvalues))
print(
    "Percentage flags on zero data: "
    + str(flagcount * 100.0 / (nch * ntimes))
    + "%"
)

# Collect statistics
# We create some unrealistic time and frequency arrays to be able
# to run these functions. Normally, these should hold the time
# and frequency values.
timeArray = numpy.linspace(0.0, ntimes, num=ntimes, endpoint=False)
freqArray = numpy.linspace(0.0, nch, num=nch, endpoint=False)
qs = aoflagger.make_quality_statistics(timeArray, freqArray, 4, False)
qs.collect_statistics(
    data, flags, aoflagger.make_flag_mask(ntimes, nch, False), 0, 1
)

try:
    qs.write_statistics("test.qs")
except:
    print("write_statistics() failed")
